# -*- coding: utf-8 -*-
import ffsend
import os
import short_url
import sys
import zipfile
try:
    from urllib import quote_plus
    from urlparse import parse_qs
except ImportError:
    from urllib.parse import quote_plus, parse_qs

import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin

ADDON_ID = xbmcaddon.Addon().getAddonInfo('id')
kodi_path = xbmc.translatePath("special://home")
build_path = xbmc.translatePath(os.path.join('special://home/addons', ADDON_ID + '/Build.zip'))
dir_path = os.path.dirname(os.path.realpath(__file__))


def categories():
    add_dir("Backup build", mode="Backup")
    add_dir("Restore build", mode="Recovery")


def add_dir(name, url='', mode='', is_folder=True, total=1):
    u = sys.argv[0] + '?url=' + quote_plus(url) + '&mode=' + str(mode) + '&name=' + quote_plus(name)
    liz = xbmcgui.ListItem(name)
    xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]), url=u, listitem=liz, isFolder=is_folder, totalItems=total)


def get_params():
    param_string = sys.argv[2]
    if param_string.startswith('?'):
        param_string = param_string[1:]
    return dict((k, vv[0]) for k, vv in parse_qs(param_string).items())


def upload_build():
    build_path = xbmc.translatePath(os.path.join('special://home/addons', ADDON_ID + '/Build.zip'))
    link, token = ffsend.upload('https://send.firefox.com/', build_path, file=None, password=None)
    link = short_url.encode_url(link)
    xbmcgui.Dialog().ok("Backup", "Your code to build: [B]%s[/B]\n\nSave and pass it during recovery (Build will expire after 1 day or 1 download)" % link)


def download_build(code):
    build_path = xbmc.translatePath(os.path.join('special://home/addons', ADDON_ID + '/Build.zip'))
    if os.path.exists(build_path):
        os.remove(build_path)
    link = short_url.decode_url(code)
    service, fid, secret = ffsend.parse_url(link)
    ffsend.download(service, fid, secret, build_path, None, url)
    return zipfile.is_zipfile(build_path)


def zip_all():
    import re
    file_count = 0
    errors = []

    excluded = [".zip", ".pyo", ".pyc", ".git", ".idea", ".log"]

    pDialog = xbmcgui.DialogProgress()
    pDialog.create('Packing', 'Please wait! Build is packing!')

    for root, dirs, files in os.walk(kodi_path):
        if pDialog.iscanceled():
            pDialog.close()
            sys.exit()
        if ADDON_ID in root:
            continue
        for file in files:
            if any(x in file or x in root for x in excluded):
                continue
            file_count += 1

    def zipdir(path, ziph, file_count):
        progress = 0
        for root, dirs, files in os.walk(path):
            if pDialog.iscanceled():
                pDialog.close()
                sys.exit()
            if ADDON_ID in root:
                continue
            for file in files:
                if any(x in file or x in root for x in excluded):
                    continue
                test = re.findall(re.escape(kodi_path) + r"(.*)", root)[0]
                progress += 1
                pDialog.update(int((progress / file_count) * 100), "", "", "")
                try:
                    ziph.write(os.path.join(root, file), test + "\\" + file)
                except:
                    errors.append(os.path.join(root, file))
                    continue
        pDialog.close()

    zipf = zipfile.ZipFile(build_path, 'w', zipfile.ZIP_DEFLATED)
    zipdir(kodi_path, zipf, file_count)
    zipf.close()
    return errors


def clear_all():
    import shutil
    for root, dirs, files in os.walk(kodi_path):
        for file in files:
            if ADDON_ID in root:
                continue
            try:
                os.remove(os.path.join(root, file))
            except:
                continue
        for dir in dirs:
            if "addons" in dir or ADDON_ID in root or ADDON_ID in dir:
                continue
            try:
                shutil.rmtree(root + "\\" + dir)
            except:
                continue


def unpack_build():
    from zipfile import ZipFile
    unpacked = 0

    pDialog = xbmcgui.DialogProgress()
    pDialog.create('Extracting', 'Please wait! Build is extracting!\n\n[B]Kodi will be closed after extracting![/B]')

    with ZipFile(build_path, 'r') as zip_ref:
        listOfFileNames = zip_ref.namelist()
        for fileName in listOfFileNames:
            if pDialog.iscanceled():
                pDialog.close()
                sys.exit()
            unpacked += 1
            pDialog.update(int((unpacked / len(listOfFileNames)) * 100), "", "", "")
            zip_ref.extract(fileName, kodi_path)
    pDialog.close()
    os._exit(1)


params = get_params()
url = params.get('url')
name = params.get('name')
try:
    mode = params.get('mode')
except:
    mode = None

if not mode:
    categories()
    xbmcplugin.setContent(int(sys.argv[1]), 'movies')
    xbmcplugin.endOfDirectory(int(sys.argv[1]))

elif mode == "Backup":
    errors = zip_all()
    if errors:
        message = ""
        for error in errors:
            message += error + "\n"
        xbmcgui.Dialog().ok("Errors", "There was not possible to copy some files:\n" + message)
    upload_build()

elif mode == "Recovery":
    keyb = xbmc.Keyboard('', "Enter build code")
    keyb.doModal()
    if keyb.isConfirmed() and len(keyb.getText().strip()) > 0:
        code = keyb.getText()
        if download_build(code):
            clear_all()
            unpack_build()
        else:
            xbmcgui.Dialog().ok("Error", "File is not a zip file!")
